/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver;

import java.io.IOException;
import java.io.InputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.apache.commons.io.IOUtils;

import se.dannej.fakehttpserver.expect.matcher.RequestMatcher;

/**
 * A {@link HttpServletRequest} that wraps the original {@link HttpServletRequest}, adding
 * a method {@link #getContent()}.
 */
public class FakeHttpServletRequest extends HttpServletRequestWrapper {
	private byte[] content;

	public FakeHttpServletRequest(HttpServletRequest request) {
		super(request);
	}
	
	/**
	 * Returns the content as a byte array. This is added to enable having more than one expectation
	 * checking the content (otherwise the first content-reading {@link RequestMatcher} would drain
	 * the request {@link InputStream}). N.B. use this instead of {@link #getInputStream()}!
	 * @return the request content
	 * @throws IOException 
	 */
	public byte[] getContent() throws IOException {
		if (content == null) {
			content = IOUtils.toByteArray(super.getInputStream());
		}
		return content;
	}

}
