/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.expect;

import static se.dannej.fakehttpserver.expect.matcher.RequestMatchers.not;
import se.dannej.fakehttpserver.FakeHttpServer;
import se.dannej.fakehttpserver.expect.cardinality.Cardinality;
import se.dannej.fakehttpserver.expect.matcher.MethodMatcher;
import se.dannej.fakehttpserver.expect.matcher.RequestMatcher;

/**
 * A helper class for creating expectations. Is called by {@link Expectations}.
 */
public class Requests {
	private final FakeHttpServer server;
	private Expectation expectation;

	Requests(FakeHttpServer server, Cardinality cardinality) {
		expectation = new Expectation();
		expectation.setcardinality(cardinality);
		this.server = server;
	}

	/**
	 * Expect a POST request.
	 * @param path
	 * @return
	 */
	public RequestExpectations post() {
		expectation.addRequestMatcher(MethodMatcher.method("POST"));
		server.addExpectation(expectation);
		return new RequestExpectations();
	}

	/**
	 * Expect a GET request.
	 * @param path
	 * @return
	 */
	public RequestExpectations get() {
		expectation.addRequestMatcher(MethodMatcher.method("GET"));
		server.addExpectation(expectation);
		return new RequestExpectations();
	}

	/**
	 * Expect a PUT request.
	 * @param path
	 * @return
	 */
	public RequestExpectations put() {
		expectation.addRequestMatcher(MethodMatcher.method("PUT"));
		server.addExpectation(expectation);
		return new RequestExpectations();
	}

	/**
	 * Expect a DELETE request.
	 * @param path
	 * @return
	 */
	public RequestExpectations delete() {
		expectation.addRequestMatcher(MethodMatcher.method("DELETE"));
		server.addExpectation(expectation);
		return new RequestExpectations();
	}

	/**
	 * Expect a HEAD request.
	 * @param path
	 * @return
	 */
	public RequestExpectations head() {
		expectation.addRequestMatcher(MethodMatcher.method("HEAD"));
		server.addExpectation(expectation);
		return new RequestExpectations();
	}

	/**
	 * Expect a OPTIONS request.
	 * @param path
	 * @return
	 */
	public RequestExpectations options() {
		expectation.addRequestMatcher(MethodMatcher.method("OPTIONS"));
		server.addExpectation(expectation);
		return new RequestExpectations();
	}

	/**
	 * Expect a request with given request method.
	 * @param method the request method to expected
	 * @param path
	 * @return
	 */
	public RequestExpectations method(String method) {
		expectation.addRequestMatcher(MethodMatcher.method(method));
		server.addExpectation(expectation);
		return new RequestExpectations();
	}

	
	void addAction(ResponseAction action) {
		expectation.addAction(action);
	}
	
	public class RequestExpectations {
		public RequestExpectations with(RequestMatcher... requestMatchers) {
			for (RequestMatcher requestMatcher : requestMatchers) {
				expectation.addRequestMatcher(requestMatcher);
			}
			return this;
		}
		
		public RequestExpectations without(RequestMatcher... requestMatchers) {
			for (RequestMatcher requestMatcher : requestMatchers) {
				expectation.addRequestMatcher(not(requestMatcher));
			}
			return this;
		}
	}
}
