/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.expect;

import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import se.dannej.fakehttpserver.FakeHttpServletRequest;
import se.dannej.fakehttpserver.FakeHttpServletResponse;
import se.dannej.fakehttpserver.JettyFakeServer;


/**
 * Invoked when none of the expectations for {@link JettyFakeServer} matched a request. 
 */
public class UnexpectedRequestExpectation extends Expectation {
	private StringBuilder message = new StringBuilder();
	private List<HttpServletRequest> unexpectedRequests = new ArrayList<HttpServletRequest>();

	@Override
	public boolean handle(FakeHttpServletRequest request, FakeHttpServletResponse response) {
		addMessage(request);
		this.unexpectedRequests.add(request);
		return true;
	}

	@Override
	public void assertIsSatisfied(Description description) {
		if (!unexpectedRequests.isEmpty()) {
			description.appendText("Unexpected request(s):\n");
			description.appendText(message.toString());
			
			fail(description.getDescription());
		}
	}

	private void addMessage(HttpServletRequest request) {
		if (message.length() > 0) {
			message.append(", ");
		}
		message.append(describeRequest(request));
	}
	
	private String describeRequest(HttpServletRequest request) {
		return String.format("%s to %s (headers: %s)", request.getMethod(), request.getRequestURI(), commaSeparated(request.getHeaderNames()));
	}
	
	private String commaSeparated(Enumeration<String> values) {
		StringBuilder sb = new StringBuilder();
		
		if (values.hasMoreElements()) {
			sb.append(values.nextElement());
			
			while (values.hasMoreElements()) {
				sb.append(", ");
				sb.append(values.nextElement());
			}
		}
		
		return sb.toString();
	}
}
