/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.expect.cardinality;

import se.dannej.fakehttpserver.expect.Description;

/**
 * Interface for classes quantifying exactly how expected a request is,
 * e.g. exactly one, never, any number of requests etc.
 */
public interface Cardinality {
	/**
	 * Called when a request matches expectations.
	 */
	void invoked();

	/**
	 * Does nothing if implementation is happy with the number of invocations,
	 * otherwise throws {@link AssertionError}.
	 * 
	 * @param description description to base assertion message on
	 */
	void assertIsSatisfied(Description description);
}
