/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.expect.matcher;

import se.dannej.fakehttpserver.FakeHttpServletRequest;
import se.dannej.fakehttpserver.expect.Description;

/**
 * {@link RequestMatcher} that matches the request method (GET, POST, etc).
 */
public class MethodMatcher implements RequestMatcher {
	private final String method;

	private MethodMatcher(String method) {
		this.method = method;
	}
	
	public String getMethod() {
		return method;
	}

	@Override
	public boolean matches(FakeHttpServletRequest request) {
		return method.equals(request.getMethod());
	}
	
	@Override
	public Description getDescription() {
		Description description = new Description();
		description.appendText(method);
		return description;
	}
	
	public static RequestMatcher method(String method) {
		return new MethodMatcher(method);
	}
}
