/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.junit;

import static se.dannej.fakehttpserver.expect.Expectations.clearExpectations;

import java.util.Arrays;
import java.util.List;

import org.junit.rules.TestRule;
import org.junit.runner.Description;
import org.junit.runners.model.Statement;

import se.dannej.fakehttpserver.FakeServer;

/**
 * This is a {@link TestRule} that shuts down any given servers
 * after executing each test.
 */
public class FakeServerRule implements TestRule {
	private final List<FakeServer> servers;

	public FakeServerRule(FakeServer... servers) {
		this.servers = Arrays.asList(servers);
	}

	public FakeServerRule(List<FakeServer> servers) {
		this.servers = servers;
	}

	@Override
	public Statement apply(final Statement base, Description description) {
		clearExpectations();
		
		return new Statement() {
			@Override
			public void evaluate() throws Throwable {
				try {
					base.evaluate();
					assertServers();
				} finally {
					shutdownServers();
				}
			}
		};
	}

	private void assertServers() {
		for (FakeServer server : servers) {
			server.assertIsSatisfied();
		}
	}

	private void shutdownServers() {
		for (FakeServer server : servers) {
			server.stop();
		}
	}
}
